from bs4 import BeautifulSoup
import requests

def get_pagesNb(soup : BeautifulSoup) -> int :
    page_toto = 0
    pages = soup.find("div",class_="showMorePages")
    if pages is not None:
        li_tag = pages.findAll("li")
        page_toto = li_tag[len(li_tag)-1].text
    else:
        pages = soup.find("nav", class_="af-pagination")
        if pages is not None:
            li_tag = pages.findAll("li")
            page_toto = li_tag[len(li_tag)-1].text
    return page_toto

def get_links(soup : BeautifulSoup) -> list[str]:
    # get all recipes from current page
    recipes = soup.find_all("div",class_="recipe-card-algolia")

    # get link of each recipe
    a_tag = []
    for recipe in recipes:
        a_tag += recipe.find_all("a",class_="recipe-card-link")
    links = []
    for a in a_tag:
        if "https://www.marmiton.org/recettes/recette_" in a["href"]:
            links += [a["href"]]
    return links

def get_title(recipe_soup : BeautifulSoup) -> str:
    title = recipe_soup.find("div", class_="main-title")
    if title is not None:
        return " ".join(title.find("h1").text.split())
    else:
        return " ".join(link[42:].split("_")[0].split("-"))
    
def get_author(recipe_soup) -> str:
    author = recipe_soup.find("span", class_="recipe-author-note__author-name")
    if author is not None:
        return author.text
    else:
        return "unknown"
    
def get_tags_and_Nutriscore(recipe_soup : BeautifulSoup) -> str:
    tag_line = ""
    tags = recipe_soup.find("div", class_="modal__tags")
    if tags is not None:
        tags_list = tags.find_all("span", class_="modal__tag") + tags.find_all("a", class_="modal__tag")
        for tag in tags_list:
            if tag is not None:
                if tag.text != "De saison" and tag.text != "Pas de saison":
                    tag_line += tag.text + ", "
        tag_line = tag_line[:-2]
    
    try:
        nutriscore = recipe_soup.find("img", class_="score-img")["alt"]
        if tag_line != "":
            tag_line += "; nutriscore : " + nutriscore
    except TypeError:
        None
    
    if tag_line != "":
        return tag_line
    else:
        return "aucun"
    
def get_time(recipe_soup : BeautifulSoup) -> str:
    time_line = ""

    time_toto = recipe_soup.find("div", class_="time__total")
    if time_toto is not None:
        if time_toto.find("span") is not None and time_toto.find("div") is not None:
            time_line += time_toto.find("span").text + time_toto.find("div").text + ", "

    time_info = recipe_soup.find("div", class_="time__details")
    if time_info is not None:
        time_info_list = time_info.find_all("div")
        for info in time_info_list:
            if info is not None:
                if info.find("span") is not None and info.find("div") is not None:
                    time_line += info.find("span").text + info.find("div").text + ", "
    
    if time_line != "":
        return time_line[:-2]
    else:
        return "aucun"
    
def get_difficulty_price(recipe_soup : BeautifulSoup) -> str:
    info_line = ""
    info = recipe_soup.find("div", class_="recipe-primary")
    if info is not None:
        info_list = info.find_all("div", class_="recipe-primary__item")[1:]
        for info_ in info_list:
            if info_ is not None:
                if info_.find("span") is not None:
                    info_line += info_.find("span").text + "; "
        return info_line[:-2]
    
    return "aucun"
    
def get_ingredients(recipe_soup : BeautifulSoup) -> str:
    ingredient_line = ""
    ingredients = recipe_soup.find("div", class_="mrtn-recette_ingredients-items")
    if ingredients is not None:
        ingredients_list = ingredients.find_all("div",class_="card-ingredient")
        for ingredient in ingredients_list:
            if ingredient is not None:
                ingredient_line += ingredient["data-name"] + ", "
        return ingredient_line[:-2]
    return "aucun"

def get_utensils(recipe_soup : BeautifulSoup) -> str:
    utensils_line = ""
    utensils = recipe_soup.find("div", class_="mrtn-recette_utensils-items")
    if utensils is not None:
        utensils_list = utensils.find_all("div", class_="card-utensil")
        for ustensil in utensils_list:
            if ustensil is not None:
                utensils_line += ustensil["data-name"] + ", "
        return utensils_line[:-2]
    return "aucun"

def get_steps(recipe_soup : BeautifulSoup) -> str:
    steps_line = ""
    steps = recipe_soup.find("div", class_="recipe-step-list")
    if steps is not None:
        steps_list = steps.find_all("p")
        for step in steps_list:
            if step is not None:
                steps_line += " ".join(step.text.split()) + " _step_ "
        return steps_line[:-8]
    return "aucun"

def update_history(data : str) -> None:
    try:
        with open("history.txt", "a") as f:
            f.write(data)
    except FileNotFoundError:
        with open("history.txt", "w") as f:
            f.write(data)

def get_recipeInfo(link):
    data = ""
    recipe_request = requests.get(link)
    if recipe_request.status_code == 200:
        print("ok, get url: " + link)
        update_history("ok, get url: " + link + "\n")

        recipe_soup = BeautifulSoup(recipe_request.text,"html5lib")

        # title
        title = get_title(recipe_soup)

        # author
        author = get_author(recipe_soup)

        # abstract : tags ans nutriscore
        categories = get_tags_and_Nutriscore(recipe_soup)

        # abstract : time
        time = get_time(recipe_soup)

        # abstract : difficulty and price
        other_info = get_difficulty_price(recipe_soup)
        
        # abstract : ingredients
        ingredients = get_ingredients(recipe_soup)

        # abstract : utensils
        utensils = get_utensils(recipe_soup)

        # abstract : steps
        steps = get_steps(recipe_soup)
        
        data += f"Marimiton, {link}\t1\t1\t1\t{title}\t{author}\ttags : {categories}; {time}; {other_info}; liste des ingredients : {ingredients}; liste des ustensiles : {utensils}; liste des instructions : {steps}\n"

    else:
        print(f"Error: {res.status_code} at url: " + link)
        update_history(f"Error: {res.status_code} at url: " + link + "\n")

    return data

def update_file(data : str) -> None:
    with open("recipes.tsv", "a") as f:
        f.write(data.replace('"',"”"))
    return

def get_recipeFileContent() -> str:
    try:
        with open("recipes.tsv", "r") as f:
            return f.read()
    except FileNotFoundError:
        data = "Source\tPublication Year\tPublication Month	Publication Day\tTitle\tAuthors\tAbstract\n"
        with open("recipes.tsv", "w") as f:
            f.write()
        update_file(data)
        return data

ingredient_list = []
with open("ingredients.tsv", 'r') as file:
    ingredient_list = file.read().split("\t")

base_url = "https://www.marmiton.org/recettes/recherche.aspx?aqt="

for ingredient in ingredient_list:
    url = base_url + ingredient
    res = requests.get(url)
    if res.status_code == 200:
        print("ok, get url: " + url)
        update_history("ok, get url: " + url + "\n")

        soup = BeautifulSoup(res.text, "html5lib")

        links = get_links(soup)
        
        # get recipe information
        for link in links:
            if link not in get_recipeFileContent():
                data = get_recipeInfo(link)
                update_file(data)

        page_toto = get_pagesNb(soup)
        if page_toto != 0:
            for i in range(1,int(page_toto)):
                new_url = url + "&page=" + str(i+1)

                new_res = requests.get(new_url)

                if new_res.status_code == 200:
                    print("ok, get url: " + new_url)
                    update_history("ok, get url: " + new_url + "\n")

                    new_soup = BeautifulSoup(new_res.text,"html5lib")

                    links = get_links(new_soup)
                    
                    # get recipe information
                    for link in links:
                        if link not in get_recipeFileContent():
                            data = get_recipeInfo(link)
                            update_file(data)
                else:
                    print(f"Error: {res.status_code} at url: " + new_url)
                    update_history(f"Error: {res.status_code} at url: " + new_url + "\n")

    else:
        print(f"Error: {res.status_code} at url: " + url)
        update_history(f"Error: {res.status_code} at url: " + url + "\n")
